/*
 * cpu.c
 */

#include <stdio.h>
#include <stdlib.h>

#include "cpu.h"
#include "memory.h"
#include "instruction.h"

/*
 *
 */

CPU *init_cpu ( CPU *cpup, int start ) {

  cpup -> pc = start;
  cpup -> acc = 0;
  cpup -> ix = 0;
  cpup -> flag = 0;			/* all clear */
  cpup -> sp = MEMORY_SIZE - 1;

  return cpup;
}

/*
 *	a++ -> 値は、今の a の値で、その値を取り出したあと、a を 1 増やす
 */

void execute ( CPU *cpup, int memory[], int debug ) {
  int ic;
  int addr;

  while ( ( ic = memory[cpup -> pc++] ) != CPU_HALT ) {

	if ( debug == 1 ) {
		printf (
				"%04x:%04d\tacc=%04d, ix=%04x, flag=%d, sp=%04x, [sp]=%04x\n",
				cpup -> pc - 1,
				ic,
				cpup -> acc,
				cpup -> ix,
				cpup -> flag,
				cpup -> sp,
				memory[ (cpup -> sp + 1) & 0xffff ] );
	}

	switch ( ic ) {
	case CPU_NOP:
	  break;
	case CPU_HALT:
	  /* 本当はここにこないが.. */
	  return;
	  break;
	case CPU_OUT:
	  putchar ( cpup -> acc );
	  break;
	case CPU_IN:
	  cpup -> acc = getchar();
	  break;
	case CPU_JUMP:
	  addr = memory[ cpup -> pc++ ];
	  cpup -> pc = addr;
	  break;
	case CPU_JZ:
	  addr = memory[ cpup -> pc++ ];
	  if ( get_zero(cpup) ) {
		cpup -> pc = addr;
	  }
	  break;
	case CPU_JN:
	  addr = memory[ cpup -> pc++ ];
	  if ( get_negative(cpup) ) {
		cpup -> pc = addr;
	  }
	  break;
	case CPU_LDI:
	  addr = memory[ cpup -> pc++ ];
	  cpup -> acc = addr;
	  break;
	case CPU_LDM:
	  addr = memory[ cpup -> pc++ ];
	  cpup -> acc = memory[addr];
	  break;
	case CPU_STM:
	  addr = memory[ cpup -> pc++ ];
	  memory[addr] = cpup -> acc;
	  break;
	case CPU_ADD:
	  addr = memory[ cpup -> pc++ ];
	  cpup -> acc += memory[addr];
	  clear_zero(cpup);
	  clear_negative(cpup);
	  if ( cpup -> acc == 0 ) {
		set_zero(cpup);
	  } else if ( cpup -> acc < 0 ) {
		set_negative(cpup);
	  }
	  break;
	case CPU_SUB:
	  addr = memory[ cpup -> pc++ ];
	  cpup -> acc -= memory[addr];
	  clear_zero(cpup);
	  clear_negative(cpup);
	  if ( cpup -> acc == 0 ) {
		set_zero(cpup);
	  } else if ( cpup -> acc < 0 ) {
		set_negative(cpup);
	  }
	  break;

	case CPU_INCX:
	  cpup -> ix++;
	  break;

	case CPU_DECX:
	  cpup -> ix--;
	  break;

	case CPU_LDXM:
	  cpup -> acc = memory[cpup -> ix];
	  break;

	case CPU_STXM:
	  memory[cpup -> ix] = cpup -> acc;
	  break;

	case CPU_LDX:
	  cpup -> acc = cpup -> ix;
	  break;

	case CPU_STX:
	  cpup -> ix = cpup -> acc;
	  break;

	case CPU_CALL:
	  addr = memory[ cpup -> pc++ ];
	  /*
			call 先のアドレスを取り出し、
			pc は、次の命令を指す
	  */
	  memory[ cpup -> sp -- ] = cpup -> pc;
	  /*
			スタックに、そのアドレスを保存
			スタックオーバーフローにきをつける
	  */
	  cpup -> pc = addr;
	  /*
			call 先に jump
	   */
	  break;

	case CPU_RET:
	  cpup -> pc = memory[ ++ cpup -> sp ];
	  /*
			スタックから、帰りアドレスを取り出し、jump
			スタックアンダーフローにきをつける
	   */
	  break;

	default:
	  fprintf ( stderr, "Illigal Instruction(%d) at %d\n", ic, cpup -> pc - 1 );
	  exit ( -1 );
	  break;
	}
  }

  return;
}

/*
 *
 */

