%{
#include <stdio.h>
#include <math.h>

#define YY_DECL int yylex()

#include "mla.tab.h"

%}

%x COMMENT
%%
[ \t]	; // ignore all whitespace
\n		{
			yylineno++;
			return T_NEWLINE;
		}
_[a-zA-Z_][a-zA-Z_0-9]*	{
	yylval.sval = strdup ( yytext );
	return T_ID;
}
[0-9]+		{
	yylval.ival = atoi(yytext);
	return T_INT;
}
0x[0-9]+		{
	sscanf ( yytext + 2, "%x", &yylval.ival );
	return T_INT;
}
'[^']'	{
	yylval.ival = yytext[1];
	return T_INT;
}
'\\.'	{
	switch ( yytext[2] ) {
	case 'n':
		yylval.ival = '\n';
		break;
	case 'r':
		yylval.ival = '\r';
		break;
	case 't':
		yylval.ival = '\t';
		break;
	case '0':
		yylval.ival = '\0';
		break;
	case '\\':
		yylval.ival = '\\';
		break;
	default:
		yylval.ival = yytext[2];
		break;
	}
	return T_INT;
}
"NOP"	{return T_NOP;}
"HALT"	{return T_HALT;}
"OUT"	{return T_OUT;}
"IN"	{return T_IN;}
"JUMP"	{return T_JUMP;}
"JZ"	{return T_JZ;}
"JN"	{return T_JN;}
"LD"	{return T_LD;}
"ST"	{return T_ST;}
"ADD"	{return T_ADD;}
"SUB"	{return T_SUB;}
"CALL"	{return T_CALL;}
"RET"	{return T_RET;}
"A"	{return T_A;}
"["	{return T_LBRA;}
"]"	{return T_RBRA;}
","	{return T_COMMA;}
":"	{return T_COLON;}
.	{return *yytext;}	/* しらないものは、上にまかせる */
"/*"                  BEGIN(COMMENT);
<COMMENT>[^\n]
<COMMENT>\n
<COMMENT><<EOF>>      yyerror("EOF in comment");
<COMMENT>"*/"         BEGIN(INITIAL);
